-- Yes yes it really is hitfeed, trust me :^)
rawset(_G, "hitfeed", true)

if HF_animIcon == nil then
	rawset(_G, "HF_animIcon", {})
end

-- validate each field in the table (this is the saving grace that'll save compatibility with mods!)
-- i dunno if it should be global but why not
rawset(_G, "HF_ValidateFields", function(table)
	if not table then return end

	-- make sure all the fields are valid in the table
	if not (table[1] and table[2])
	or not (table[1][1] and table[1][2])	-- name and name colour
	or not (table[2][1] and table[2][2])	-- icon and icon colour
		return nil
	end

	return table
end)

local ICONHEIGHT = HL_ICONHEIGHT
local ICONOFFSET = HL_ICONOFFSET

local function set(arr)
	local s = {}
	for _, value in ipairs(arr) do
		s[value] = true
	end
	return s
end

-- So, hitfeed works even if disabled, but we can't just check for that since
-- doing so will disable custom messages. To fix this we will check only for
-- "vanilla" hitfeed messages
local HFVANILLA = set {
	"K_HMBNA",
	"K_HMEGG",
	"K_HMHYU",
	"K_HMINV",
	"K_HMMNE",
	"K_HMSNK",
	"K_HMSPB",
	"K_HMSRK",
	"K_HMBNA1",
	"K_HMEGG1",
	"K_HMHYU1",
	"K_HMINV",
	"K_HMMNE1",
	"K_HMSNK",
	"K_HMSPB1",
	"K_HMORBN",
	"K_HMJAWZ",
	"K_HMBHOG",
	"K_HMTHNS",
	"K_HMRSHE",
	"K_HMSHOE",
	"K_HMPOGO",
	"K_HMGROW",
	"K_HMPANK",
}

local oldHF_SendHitMessage = HF_SendHitMessage

-- function to send a hitmessage
rawset(_G, "HF_SendHitMessage", function(s, v, icon, color) -- source player, victim player, icon, color (optional)
	--(void)color; :^) (ye its unused for now)
	-- checks before going through with it
	
	-- Hitlist is disabled, try using original hitfeed function
	if not cv_hl.value then
		if oldHF_SendHitMessage then
			oldHF_SendHitMessage(s, v, icon, color)
		end
		
		return
	end
	
	-- Hitlist has more or less same messages as vanilla hitfeed, so we can omit those
	-- Also, did you know that hitfeed works even when it's disabled?
	if HFVANILLA[icon] and cv_hl.value then return end
	if not (icon) then return end -- if it has no icon we cant do this

	s = HF_ValidateFields(s)
	v = HF_ValidateFields(v)

	if not (v and v[3]) then return end -- target can't be nil

	local hit = {
		target = v[3],
		inflictor = nil,
		source = s and s[3],
		icon = icon,
		manualdraw = true,
		extrafunc = function(v, hit, ofs, hx, hy, vflags)
			local icon_name = hit.icon
			
			if HF_animIcon[icon_name] then
				local anim = HF_animIcon[icon_name]
				
				local frame = ((leveltime / anim[2]) % anim[1]) + 1
				
				icon_name = icon_name..frame
			end
			
			local multiheight = ofs.multiheight
		
			local icon = v.cachePatch(icon_name)
			local scale = FixedDiv(ICONHEIGHT, max(ICONHEIGHT, icon.height))
			v.drawScaled((hx + ofs.icon)<<FRACBITS, (hy + multiheight/2)<<FRACBITS + ICONOFFSET, scale, icon, vflags)
		end
	}
	
	HL_AddHit(hit)
end)